package aderby;

import aderby.jdkjre.compilation4.AriaClassLoader;
import org.apache.derby.drda.NetworkServerControl;

import java.io.*;
import java.lang.reflect.InvocationTargetException;
import java.net.InetAddress;
import java.nio.file.*;
import java.nio.file.attribute.BasicFileAttributes;
import java.sql.*;
import java.util.*;

public class DerbyUtil {
    public static final String embdriver = "EmbeddedDriver";
    public static final String clidriver = "ClientDriver";
    public static final String lhost = "localhost";
    public static final int port = 1527;
    public static final String separator = System.getProperty("file.separator");
    public static final String createTable1 = "CREATE TABLE klasy(plik VARCHAR(256), klasa BLOB)";
    public static final String createTableSource = "CREATE TABLE klasy(plik VARCHAR(256), klasa CLOB)";
    private DerbyUtil() {
    }
    //---------------------- R007 ------------------------------------------

    /**
     * Startuje DerbyEngine. Jeśli załadowano Embedded Driver można
     * od razu połączyć się z bazą danych. Jeśli załadowano ClientDriver
     * aby połączyć się z bazą danych należy wcześniej uruchomić NetworkServer
     * na tym lub zdalnym komputerze (w zależności gdzie znajduje się baza
     * danych). Jeśli baza danych znajduje się na innym komputerze to
     * połączenie z nią jest możliwe o ile na komputerze z bazą
     * zostaną ustawione odpowiednie prawa dostępu. Jeżeli nie zostaną
     * ustawione możliwe jest połączenie tylko z tego komputera, na którym
     * baza się znajduje
     *
     * @param driverName - nazwa sterownika
     * @return - true -jeśli sterownik został załadowany i silnik bazy danych
     * Derby został uruchomiony, false - jeśli sterownik bazy danych nie został
     * załadowany i silnik bazy danych nie został uruchomiony
     */
    public static boolean startDerbyEngine(String driverName) {
        return loadDBDriver(driverName);
    }

    /**
     * Ładuje wskazany sterownik bazy danych
     *
     * @param driverName - nazwa sterownika do załadowania
     * @return - true - jeśli sterownik został załadowany, false - w
     * przeciwnym przypadku
     * @throws IllegalArgumentException - wyrzucany wyjątek
     */
    private static boolean loadDBDriver(String driverName)
            throws IllegalArgumentException {
        if (driverName.equals("EmbeddedDriver")
                || driverName.equals("ClientDriver")) {
            try {
                try {
                    Class.forName("org.apache.derby.jdbc." + driverName).
                            getDeclaredConstructors()[0].newInstance();
                } catch (InstantiationException | IllegalAccessException
                         | InvocationTargetException e) {
                    e.printStackTrace();
                }
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
                return false;
            }
        } else {
            throw new IllegalArgumentException("Niewłaściwa nazwa sterownika");
        }
        return true;
    }

    /**
     * Tworzy połączenie z wbudowaną bazą danych. Ustawia właściwości określone
     * w łańcuchu znaków 'properties'
     *
     * @param dbName     - nazwa bazy danych lub ścieżka do bazy danych z nazwą
     *                   bazy danych na końcu
     * @param properties - pary właściwości połączenia (nazwa=właściwość)
     *                   oddzielone średnikami. Średnik na początku i końcu nie są potrzebne.
     * @return - połączenie z bazą danych
     */
    public static Connection connectEmbeddedDB(String dbName, String properties) {
        Connection conn = null;
        try {
            conn = DriverManager.getConnection("jdbc:derby:" + dbName + ";" + properties);
        } catch (Throwable e1) {
            e1.printStackTrace();
        }
        return conn;
    }

    /**
     * Zamyka polecenie
     *
     * @param stat - polecenie do zamknięcia
     */
    public static void close(Statement stat) {
        if (stat != null) {
            try {
                stat.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Zamyka połączenie
     *
     * @param conn - połączenie do zamknięcia
     */
    public static void close(Connection conn) {
        if (conn != null) {
            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Zamyka bazę o podanej nazwie
     *
     * @param dbName - nazwa bazy do zamknięcia
     * @return true - jeśli baza została zamknięta, false - w przeciwnym wypadku
     */
    public static boolean shutdownEmbeddedDB(String dbName) {
        boolean gotSQLExc = false;
        try {
            DriverManager
                    .getConnection("jdbc:derby:" + dbName + ";shutdown=true");
        } catch (SQLException se) {
            if (se.getSQLState().equals("XJ015")) {
                gotSQLExc = true;
            }
        }
        return gotSQLExc;
    }

    /**
     * Zatrzymuje silnik bazy danych Derby i wszystkie bazy danych,
     *
     * @return - true - jeśli silnik został zatrzymany i wszystkie bazy zostały
     * zamknięte, false - w przeciwnym przypadku.
     */
    public static boolean shutdownDerbyEngine() {
        boolean gotSQLExc = false;
        try {
            DriverManager.getConnection("jdbc:derby:;shutdown=true");
        } catch (SQLException se) {
            if (se.getSQLState().equals("XJ015")) {
                gotSQLExc = true;
            }
        }
        return gotSQLExc;
    }

    /**
     * Zamyka zbiór wynikowy
     *
     * @param rs - zamykany zbiór wynikowy
     */
    public static void close(ResultSet rs) {
        if (rs != null) {
            try {
                rs.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }
    //---------------------- R016 ----------------------------------------

    /**
     * Uruchamia server Derby
     *
     * @param host - adres serwera
     * @param port - port w komputerze, na którym działa serwer
     */
    public static void startDerbyNetworkServer(String host, int port) {
        NetworkServerControl server;
        try {
            server = new NetworkServerControl(InetAddress.getByName(host), port);
            server.start(null);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Bez autentykacji użytkownika (bez podawania użytkownika i hasła)
     * Zamyka serwer i wszystkie bazy danych na serwerze
     */
    public static void shutdownDerbyNetworkServer() {
        NetworkServerControl server;
        try {
            server = new NetworkServerControl();
            server.shutdown();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Łączy z bazą danych poprzez server
     *
     * @param dbName - nazwa bazy danych
     * @param host   - adres serwera
     * @param port   - port komputera na którym działa server
     * @param props  - właściwości połączenia. Do właściwości można dołączyć
     *               nazwę użytkownika i hasło
     * @return - true - jeśli połączenie zostało nawiązane, false - w
     * przeciwnym przypadku
     */
    public static Connection connectServerDB(String dbName, String host, String port, String props) {
        Connection conn = null;
        String url = "jdbc:derby:" + "//" + host + ":" + port + "/" + dbName + ";" + props;
        try {
            conn = DriverManager.getConnection(url);
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return conn;
    }

    /**
     * Zamyka pojedynczą bazę danych na serwerze
     *
     * @param dbName - nazwa bazy
     * @param host-  użytkownik
     * @param port   - hasło
     * @param props  - właściwości
     * @return - true - jeśli baza została zamknięta, false - w przeciwnym
     * przypadku.
     */
    public static boolean shutdownServerDB(String dbName, String host, int port, String props) {
        boolean gotSQLExc = false;
        try {
            DriverManager.getConnection("jdbc:derby://" + host + ":" + port
                    + "/" + dbName + ";" + props);
        } catch (SQLException se) {
            if (se.getSQLState().equals("08006")) {
                gotSQLExc = true;
            }
        }
        return gotSQLExc;
    }

    //-------------------- R019 ------------------------------------
    public static void close(OutputStream rs) {
        if (rs != null) {
            try {
                rs.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public static void close(InputStream rs) {
        if (rs != null) {
            try {
                rs.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Ładuje właściwości konfiguracyjne dla bazy danych.
     * Każda z właściwości w pliku musi być we własnej linijce.
     * Każda linijka to para nazwa=wartość, np.: user=Jacek.
     */
    public static Properties loadProperties(String file) {
        FileInputStream fis = null;
        Properties props;
        try {
            fis = new FileInputStream(file);
        } catch (FileNotFoundException e1) {
            e1.printStackTrace();
        }
        props = new Properties();
        try {
            props.load(Objects.requireNonNull(fis));
        } catch (IOException e) {
            e.printStackTrace();
        }
        DerbyUtil.close(fis);
        return props;
    }

    //---------------------------- R020 --------------------------------
    public static void close(Writer wr) {
        if (wr != null) {
            try {
                wr.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public static void close(Reader re) {
        if (re != null) {
            try {
                re.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
    //-------------------R023 ---------------------------------------

    /**
     * ustawia podkatalog dla bazy danych. Jeżeli katalogiem
     * jest "../Moje dokumenty" to katalogiem dla bazy danych
     * będzie "../Moje dokumenty/.dirName/"
     */
    public static void setDBSystemDir(String dirName) {
        String userHomeDir = System.getProperty("user.home", ".");
        String systemDir = userHomeDir + "/." + dirName;
        System.setProperty("derby.system.home", systemDir);
    }

    /**
     * Tworzy połączenie z wbudowaną bazą danych. Ustawia właściwości określone
     * w obiekcie 'props' klasy Properties
     *
     * @param dbName - nazwa bazy danych lub ścieżka do bazy danych z nazwą
     *               bazy danych na końcu
     * @param props  - Obiekt klasy Properties zawierający właściwości
     *               połączenia do ustawienia.
     * @return - połączenie z bazą danych
     */
    public static Connection connectEmbeddedDB(String dbName, Properties props) {
        Connection conn = null;
        try {
            conn = DriverManager.getConnection("jdbc:derby:" + dbName, props);
        } catch (Throwable e1) {
            e1.printStackTrace();
        }
        return conn;
    }
    //------------------------- R025 ------------------------------------

    /**
     * Tworzy połączenie z wbudowaną bazą danych. Nie są ustawiane żadne
     * właściwości
     *
     * @param dbName - nazwa bazy danych lub ścieżka do bazy danych i nazwą
     *               bazy danych na końcu
     * @return - połączenie do bazy danych;
     */
    public static Connection connectEmbeddedDB(String dbName) {
        Connection conn = null;
        try {
            conn = DriverManager.getConnection("jdbc:derby:" + dbName);
        } catch (Throwable e1) {
            e1.printStackTrace();
        }
        return conn;
    }
    //---------------------------- R026 -----------------------------

    /**
     * Zamyka bazę danych o podanej nazwie z użyciem właściwości w postaci
     * obiektu props klasy properties
     *
     * @param dbName - nazwa bazy do zamknięcia
     * @param props  - właściwości połączenia do zastosowania
     * @return - true - jeśli baza została zamknięta, false - w przeciwnym
     * wypadku
     */
    public static boolean shutdownEmbeddedDB(String dbName, Properties props) {
        boolean gotSQLExc = false;
        try {
            DriverManager.getConnection("jdbc:derby:" + dbName, props);
        } catch (SQLException se) {
            if (se.getSQLState().equals("XJ015")) {
                gotSQLExc = true;
            }
        }
        return gotSQLExc;
    }

    /**
     * Zamyka bazę danych o podanej nazwie z użyciem properties w postaci
     * łańcucha znaków
     *
     * @param dbName     - nazwa bazy do zamknięcia
     * @param properties - właściwości połączenia do użycia
     * @return - true - jeśli baza została zamknięta, false - w przeciwnym
     * wypadku
     */
    public static boolean shutdownEmbeddedDB(String dbName, String properties) {
        boolean gotSQLExc = false;
        try {
            DriverManager
                    .getConnection("jdbc:derby:" + dbName + ";" + properties);
        } catch (SQLException se) {
            if (se.getSQLState().equals("XJ015")) {
                gotSQLExc = true;
            }
        }
        return gotSQLExc;
    }

    // ---------------------------------------- R027 -------------------------

    /**
     * Usuwa bazę danych (folder bazy danych)
     *
     * @param dbPath - ścieżka do folderu bazy danych do usunięcia
     * @return - true - jeśli baza danych została usunięta, false - w
     * przeciwnym wypadku.
     */
    public static boolean dropDatabase(File dbPath) {
        if (dbPath == null) {
            return false;
        }
        if (dbPath.isDirectory()) {
            String[] dirList = dbPath.list();
            int len = Objects.requireNonNull(dirList).length;
            for (String s : dirList) {
                if (!dropDatabase(new File(
                        dbPath + separator + s))) {
                    return false;
                }
            }
        }
        if (!dbPath.delete()) {
            System.out.println("Nie mogę usunąć: " + dbPath.getAbsolutePath());
            return false;
        }
        return true;
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    private static void deleteDirs(File dbPathAndName) {
        File[] dirs = dbPathAndName.listFiles();
        for (File dir : Objects.requireNonNull(dirs)) {
            dir.delete();
        }
        dbPathAndName.delete();
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    private static void deleteFiles(File file) {
        if (file.isDirectory()) {
            File[] files = file.listFiles();
            for (File value : Objects.requireNonNull(files)) {
                deleteFiles(value);
            }
        } else {
            file.delete();
        }
    }

    public static void dropDatabase(String dbPathAndName) {
        File baza = new File(dbPathAndName);
        deleteFiles(baza);
        deleteDirs(baza);
        File der = new File("derby.log");
        //noinspection ResultOfMethodCallIgnored
        der.delete();
    }

    /**
     * Usuwa bazę danych z pamięci
     *
     * @param dbName - nazwa bazy
     */
    public static void dropInMemoryDB(String dbName) {
        try {
            DriverManager.getConnection(
                    "jdbc:derby:memory:" + dbName + ";drop=true");
        } catch (SQLException se) {
            se.printStackTrace();
        }
    }

    public static void deleteDB(String path) throws IOException {
        Path start = Paths.get(path);
        Files.walkFileTree(start, new SimpleFileVisitor<>() {
            @Override
            public FileVisitResult visitFile(Path file,
                                             BasicFileAttributes attrs) throws IOException {
                Files.delete(file);
                return FileVisitResult.CONTINUE;
            }

            @Override
            public FileVisitResult postVisitDirectory(Path dir, IOException e)
                    throws IOException {
                if (e == null) {
                    Files.delete(dir);
                    return FileVisitResult.CONTINUE;
                } else {
                    throw e;
                }
            }
        });
    }

    public static void copyDB(String pathIn, String pathOut) throws IOException {
        final Path source = Paths.get(pathIn);
        final Path target = Paths.get(pathOut);
        Files.walkFileTree(source, EnumSet.of(FileVisitOption.FOLLOW_LINKS),
                Integer.MAX_VALUE, new SimpleFileVisitor<>() {
                    @Override
                    public FileVisitResult preVisitDirectory(Path dir,
                                                             BasicFileAttributes attrs) throws IOException {
                        Path targetdir = target.resolve(source.relativize(dir));
                        try {
                            Files.copy(dir, targetdir);
                        } catch (FileAlreadyExistsException e) {
                            if (!Files.isDirectory(targetdir)) {
                                throw e;
                            }
                        }
                        return FileVisitResult.CONTINUE;
                    }

                    @Override
                    public FileVisitResult visitFile(Path file,
                                                     BasicFileAttributes attrs) throws IOException {
                        Files.copy(file,
                                target.resolve(source.relativize(file)));
                        return FileVisitResult.CONTINUE;
                    }
                });
    }

    //---------------------------- R036 -----------------------------------

    /**
     * Uruchamia skrypt SQL. Skrypt może być wykonany z poziomu JDBC jak tutaj
     * albo z poziomu narzędzia ij
     *
     * @param scriptFile - sciezka do skryptu, który należy uruchomic
     * @param con        - połączenie z bazą, wykonującą zadanie.
     */
    public static void runScript(String scriptFile, Connection con) {
        FileInputStream fileStream = null;
        try {
            fileStream = new FileInputStream(scriptFile);
            int result = org.apache.derby.tools.ij.runScript(con, fileStream, "UTF-8", System.out,
                    "UTF-8");
            //System.out.println("Result code is: " + result);
        } catch (FileNotFoundException | UnsupportedEncodingException e) {
            e.printStackTrace();
        } finally {
            close(fileStream);
        }
    }
    //------------------------- R041 ------------------------------------

    /**
     * Podaje losową liczbę z przedziału zamkniętego <min, max>
     *
     * @param min long - najmniejsza wartość
     * @param max long - największa wartość
     * @return long - wylosowana wartość z podanego zakresu, z uwzględnieniem min i
     * max
     */
    public static long randomInRange(long min, long max) {
        long random = -1;
        if (min > max) {
            System.out.println("pierwsza liczba musi byc mniejsza od drugiej");
        } else {
            random = (long) (Math.floor(Math.random() * (max - min + 1)) + min);
        }
        return random;
    }

    /**
     * zwraca pierwszą liczbę pierwszą większą od podanej liczby
     *
     * @param min int - liczba progowa
     * @return int - pierwsza liczba pierwsza większa od min
     */
    public static long getPrime(long min) {
        for (long j = min + 1; true; j++) {
            if (isPrime(j)) {
                return j;
            }
        }
    }

    /**
     * Sprawdza czy podana liczba jest liczbą pierwszą
     *
     * @param n - badana liczba
     * @return - true jeśli badana liczba jest liczbą pierwszą, false
     * w przeciwnym przypadku
     */
    public static boolean isPrime(long n) {
        for (long j = 2; Math.pow(j, 2) <= n; j++) {
            if (n % j == 0) {
                return false;
            }
        }
        return true;
    }

    //-------------------------- R046 ---------------------------------------
    private static String scriptToString(String file) {
        FileReader fr = null;
        BufferedReader br = null;
        StringBuilder sb = new StringBuilder();
        String line;
        try {
            fr = new FileReader(file);
            br = new BufferedReader(fr);
            while ((line = br.readLine()) != null) {
                sb.append(line);
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            close(br);
            close(fr);
        }
        return sb.toString();
    }

    private static ArrayList<String> tokenize(String text) {
        ArrayList<String> al = new ArrayList<>();
        StringTokenizer st = new StringTokenizer(text, ";", false);
        while (st.hasMoreTokens()) {
            al.add(st.nextToken());
        }
        return al;
    }

    public static void jdbcRunScript(String file, Connection conn) {
        String b = scriptToString(file);
        ArrayList<String> al = tokenize(b);
        Statement stat = null;
        try {
            stat = conn.createStatement();
            for (String a : al) {
                stat.addBatch(a);
            }
            stat.executeBatch();
        } catch (SQLException e1) {
            e1.printStackTrace();
        }
        DerbyUtil.close(stat);
    }
    //------------------------ R052 ---------------------------------

    /**
     * Baza danych "baza", tabela "obrazki" utworzona
     * poleceniem "createTableObrazki"
     * Wczytuje wszystkie obrazki o podanym rozszerzeniu z podanego katalogu
     * dir - folder z obrazkami
     * ext rozszerzenie nazwy pliku np. ".png", ".jpg"
     */
    public static void imagesToDerby(Connection con, String dir, String ext) {
        PreparedStatement pstm = null;
        File file;
        long length;
        InputStream in = null;
        String[] obrazki = DerbyUtil.listFileNames(dir, ext);
        try {
            pstm = con.prepareStatement("INSERT INTO obrazki VALUES(DEFAULT,?,?)");
        } catch (SQLException e1) {
            e1.printStackTrace();
        }
        for (String s : obrazki) {
            file = new File(dir + "\\" + s);
            length = file.length();
            try {
                in = new FileInputStream(file);
            } catch (FileNotFoundException e1) {
                e1.printStackTrace();
            }
            try {
                Objects.requireNonNull(pstm).setString(1, s);
                pstm.setBlob(2, in, length);
                pstm.execute();
                pstm.clearParameters();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
        DerbyUtil.close(pstm);
        close(in);
    }

    /**
     * tworzy listę plików typu "*.jpg"  w podanym katalogu
     *
     * @param dir String - katalog zawierający pliki
     * @param ext rozszerzenie z kropką np. ".jpg"
     * @return String[] - lista plików w podanym katalogu
     */
    public static String[] listFileNames(String dir, String... ext) {
        File file = new File(dir);
        File[] files = null;
        String[] names;
        if (file.isDirectory()) {
            MultiFilter fef = new MultiFilter(file, ext);
            files = file.listFiles(fef);
        } else {
            System.out.println("Podaj nazwę katalogu z plikami");
        }
        names = new String[Objects.requireNonNull(files).length];
        for (int i = 0; i < files.length; i++) {
            names[i] = files[i].getName();
        }
        return names;
    }

    /**
     * Kopiuje strumień wejściowy do wyjściowego
     *
     * @param in  - strumień wejściowy
     * @param out - strumień wyjściowy
     */
    public static void copyStream2(InputStream in, OutputStream out) {
        byte[] buffer = new byte[1024];
        while (true) {
            int bytesRead;
            try {
                bytesRead = in.read(buffer);
                if (bytesRead == -1) {
                    break;
                }
                out.write(buffer, 0, bytesRead);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
    //-------------------- R053 ---------------------------------

    /**
     * Zamienia tblicę bajtów na liczby heksadecymalne
     *
     * @param bytes - tablica bajtów
     * @return - Ciąg znaków heksadecymalnych
     */
    public static String byteArrayToHexString(byte[] bytes) {
        StringBuilder sb = new StringBuilder();
        for (byte b : bytes) {
            sb.append(String.format("%02x", b));
        }
        return sb.toString();
    }

    public static <T> void print(T[] array) {
        System.out.print("[");
        for (int i = 0; i < array.length; i++) {
            if (i == array.length - 1) {
                System.out.print(array[i]);
            } else {
                System.out.print(array[i] + ", ");
            }
        }
        System.out.print("]");
        System.out.println();
    }

    public static <T> void print(T obj) {
        System.out.println(obj);
    }
//------------------ R086 ----------------------------------------------

    /**
     * oblicza logarytm o dowolnej podstawie z podanej liczby
     *
     * @param liczba   double - liczba, której logarytm trzeba obliczyć
     * @param podstawa double - podstawa logarytmu
     * @return double - obliczony logarytm
     */
    public static double lg(double liczba, int podstawa) {
        return Math.log10(liczba) / (Math.log10(podstawa));
    }
    //---------------------------- R087 ------------------------------

    /**
     * oblicza potege dowolnego stopnia
     */
    public static double pow(double liczba, double potega) {
        return Math.pow(liczba, potega);
    }
    //--------------------------------- R088 ------------------------

    /**
     * funkcja zaokragla podaną liczbę do określonej liczby miejsc po przecinku
     *
     * @param num double - liczba do zaokrąglenia
     * @param dec int - liczba miejsc po przecinku;
     * @return double - zwraca zaokrągloną liczbę
     */
    public static double roundToDecimal(double num, int dec) {
        int multi = (int) Math.pow(10, dec);
        int temp = (int) Math.round(num * multi);
        return (double) temp / multi;
    }

    ///------------------------------- R089 ----------------------------------

    /**
     * oblicza pierwiastek dowolnego stopnia
     */
    public static double root(double liczba, double stopien) {
        return Math.pow(liczba, 1.0 / stopien);
    }

    //---------------------------- R090 --------------------------------------

    /**
     * funkcja XOR = "albo to albo to" = alternatywa wyłączająca
     * = nierównoważność
     *
     * @return zwraca false jeśli p, q sa jednakowe, true - jesli
     * sa rozne
     */
    public static boolean xor(boolean p, boolean q) {
        return (p || q) && !(p && q);
    }

    //------------------------------ R092 ----------------------------------

    /**
     * Łączy z bazą danych w pamięci
     *
     * @param dbName     - nazwa bazy
     * @param properties - właściwości
     * @return - połączenie z bazą
     */
    public static Connection connectMemoryDB(String dbName, String properties) {
        Connection conn = null;
        try {
            conn = DriverManager.getConnection(
                    "jdbc:derby:memory:" + dbName + ";" + properties);
        } catch (Throwable e1) {
            e1.printStackTrace();
        }
        return conn;
    }
    //-------------------------- R093 --------------------------------------

    public static void setReadOnlyProps() {
        System.setProperty("derby.storage.tempDirectory", "C:/temp/mytemp");
        System.setProperty("derby.stream.error.file", "C:/temp/mylog.log");
    }

    //------------------------ R095 -------------------------------------
    public static Connection connectJarDB(String dbName, String properties) {
        Connection conn = null;
        try {
            conn = DriverManager.getConnection(
                    "jdbc:derby:jar:" + dbName + ";" + properties);
        } catch (Throwable e1) {
            e1.printStackTrace();
        }
        return conn;
    }

    //------------------------- R096 -------------------------------------
    public static Connection connectClasspathDB(String dbName,
                                                String properties) {
        Connection conn = null;
        try {
            conn = DriverManager.getConnection(
                    "jdbc:derby:classpath:" + dbName + ";" + properties);
        } catch (Throwable e1) {
            e1.printStackTrace();
        }
        return conn;
    }
    //-------------------- R097 -----------------------------------------

    /**
     * Zamienia liczbę HEX na RGB
     *
     * @param hexa String - liczba typu 0xFFFFFF podawana jest jako
     *             "FFFFFF" bez 0x
     * @return int[] - zawierający kolejno r, g, b
     */
    public static int[] HexToRGB(String hexa) {
        Integer r = Integer.valueOf(hexa.substring(0, 2), 16);
        Integer g = Integer.valueOf(hexa.substring(2, 4), 16);
        Integer b = Integer.valueOf(hexa.substring(4, 6), 16);
        return new int[]{r, g, b};
    }

    /**
     * Zamienia liczbę HEX na RGB
     *
     * @param hexa int - podajemy liczbę typu 0xFFFFFF
     * @return int[] zawierający kolejno r, g, b
     */
    public static int[] HexToRGB(int hexa) {
        int r = (hexa >> 16) & 0xff;
        int g = (hexa >> 8) & 0xff;
        int b = hexa & 0xff;
        return new int[]{r, g, b};
    }

    /**
     * Zamienia wartosci R, G, B na liczbę HEX
     *
     * @param r int - składnik czerwony
     * @param g int - składnik zielony
     * @param b int - składnik niebieski
     * @return String - wartość HEX bez "0x" na początku
     */
    public static String RGBToHex(int r, int g, int b) {
        String r1 = Integer.toString(r, 16);
        String g1 = Integer.toString(g, 16);
        String b1 = Integer.toString(b, 16);
        if (r1.length() == 1) {
            r1 = "0" + r1;
        }
        if (g1.length() == 1) {
            g1 = "0" + g1;
        }
        if (b1.length() == 1) {
            b1 = "0" + b1;
        }
        return r1.concat(g1).concat(b1);
    }

    /**
     * Zamienia wartości R, G, B na liczbę int stosowaną w Javie
     *
     * @param r int - składnik czerwony;
     * @param g int - składnik zielony
     * @param b int - składnik niebieski
     * @return int - wartość RGB przeliczona na int;
     */
    public static int RGBToInt(int r, int g, int b) {
        String r1 = Integer.toString(r, 16);
        String g1 = Integer.toString(g, 16);
        String b1 = Integer.toString(b, 16);
        if (r1.length() == 1) {
            r1 = "0" + r1;
        }
        if (g1.length() == 1) {
            g1 = "0" + g1;
        }
        if (b1.length() == 1) {
            b1 = "0" + b1;
        }
        String cont = "" + r1 + g1 + b1;
        int i = Integer.valueOf(cont, 16);
        return (i | 0x00 << 16 | 0x00 << 8 | 0x00);
    }

    /**
     * Przelicza liczbę int na R, G, B
     *
     * @param rgb int - liczba int oznaczająca kolor;
     * @return String - wartość RGB
     */
    public static int[] IntToRGB(int rgb) {
        int r = (rgb >> 16) & 0xff;
        int g = (rgb >> 8) & 0xff;
        int b = (rgb >> 0) & 0xff;
        return new int[]{r, g, b};
    }

    public static boolean installJar(Connection conn, String jarPath,
                                     String jarName, int deploy) {
        CallableStatement cstat = null;
        boolean done = false;
        try {
            cstat = conn.prepareCall("CALL SQLJ.INSTALL_JAR(?,?,?)");
            cstat.setString(1, jarPath);
            cstat.setString(2, jarName);
            cstat.setInt(3, deploy);
            done = cstat.execute();
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            DerbyUtil.close(cstat);
        }
        return done;
    }

    public static void print(int[] array) {
        System.out.print("[");
        for (int i = 0; i < array.length; i++) {
            if (i == array.length - 1) {
                System.out.print(array[i]);
            } else {
                System.out.print(array[i] + ", ");
            }
        }
        System.out.print("]");
        System.out.println();
    }

    public static boolean replaceJar(Connection conn, String newJar,
                                     String oldJarDBName) {
        CallableStatement cstat = null;
        boolean done = false;
        try {
            cstat = conn.prepareCall("CALL SQLJ.REPLACE_JAR(?,?)");
            cstat.setString(1, newJar);
            cstat.setString(2, oldJarDBName);
            done = cstat.execute();
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            DerbyUtil.close(cstat);
        }
        return done;
    }

    public static boolean removeJar(Connection conn, String jarDBName,
                                    int undeploy) {
        CallableStatement cstat = null;
        boolean done = false;
        try {
            cstat = conn.prepareCall("CALL SQLJ.REMOVE_JAR(?,?)");
            cstat.setString(1, jarDBName);
            cstat.setInt(2, undeploy);
            done = cstat.execute();
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            DerbyUtil.close(cstat);
        }
        return done;
    }

    /**
     * Ustawia pojedynczą właściwość w bazie danych
     *
     * @param conn     - połączenie
     * @param property - nazwa właściwości
     * @param value    - ustawiana wartość
     * @return - true - jeśli właściwość została ustawiona, false - w
     * przeciwnym wypadku
     */
    public static boolean setDBProperty(Connection conn, String property,
                                        String value) {
        CallableStatement cstat = null;
        boolean done = false;
        try {
            cstat = conn.prepareCall(
                    "CALL SYSCS_UTIL.SYSCS_SET_DATABASE_PROPERTY(?,?)");
            cstat.setString(1, property);
            cstat.setString(2, value);
            done = cstat.execute();
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            close(cstat);
        }
        return done;
    }
    // ----------------------- R098 -------------------------------------
    public static String buildSourceString() {
        String str;
        StringWriter sw = new StringWriter();
        sw.append("public class HelloWorld {");
        sw.append("  public static void main(String args[]) {");
        sw.append("    System.out.println(\"Hello, World\");");
        sw.append("  }");
        sw.append("}");
        sw.flush();
        str = sw.toString();
        close(sw);
        return str;
    }

    public static String buildSourceString2() {
        String str;
        StringWriter sw = new StringWriter();
        sw.append("public class JPHelloWorld {");
        sw.append("  public static void main(String args[]) {");
        sw.append("    System.out.println(\"Hello, World\");");
        sw.append("  }");
        sw.append("}");
        sw.flush();
        str = sw.toString();
        close(sw);
        return str;
    }

    public static void classesToDerby(String dir) {
        DerbyUtil.startDerbyNetworkServer("localhost", 1527);
        DerbyUtil.startDerbyEngine("ClientDriver");
        Connection con = DerbyUtil.connectServerDB("klasy", "localhost", "1527",
                "create=true");
        Statement stat = null;
        try {
            stat = con.createStatement();
            stat.executeUpdate(DerbyUtil.createTable1);
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            DerbyUtil.close(stat);
        }
        PreparedStatement pstm = null;
        File file;
        long length;
        BufferedInputStream in = null;
        String[] klasy = listFileNames("aderby/src/resources/compilation/c2/", ".class");
        try {
            pstm = con.prepareStatement("INSERT INTO klasy VALUES(?,?)");
            for (String s : klasy) {
                int j = s.indexOf(".");
                file = new File("aderby/src/resources/compilation/c2/" + s);
                try {
                    in = new BufferedInputStream(new FileInputStream(file));
                } catch (FileNotFoundException e1) {
                    e1.printStackTrace();
                }
                length = file.length();
                try {
                    pstm.setString(1, s.substring(0, j));
                    pstm.setBlob(2, in, length);
                    pstm.execute();
                    //pstm.clearParameters();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        //-
        DerbyUtil.close(pstm);
        DerbyUtil.close(in);
        DerbyUtil.close(con);
        DerbyUtil.shutdownDerbyEngine();
        DerbyUtil.shutdownDerbyNetworkServer();
    }

    public static LinkedHashMap<String, byte[]> classesFromDerby() {
        DerbyUtil.startDerbyNetworkServer("localhost", 1527);
        DerbyUtil.startDerbyEngine("ClientDriver");
        Connection con = DerbyUtil.connectServerDB("klasy", "localhost", "1527",
                "");
        Statement stat = null;
        ResultSet rs = null;
        //InputStream in = null;
        LinkedHashMap<String, byte[]> klasy = new LinkedHashMap<>();
        try {
            stat = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
                    ResultSet.CONCUR_READ_ONLY);
            rs = stat.executeQuery("SELECT * FROM klasy");
            rs.beforeFirst();
            while (rs.next()) {
                String str = rs.getString("plik");
                Blob blob = rs.getBlob("klasa");
                //in = blob.getBinaryStream();
                //byte[] bajty = rs.getBytes("klasa");
                byte[] bajty = blob.getBytes(1, (int) blob.length());
                klasy.put(str, bajty);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            DerbyUtil.close(stat);
            DerbyUtil.close(rs);
            //IOUtil.close(in);
        }
        DerbyUtil.close(con);
        DerbyUtil.shutdownDerbyEngine();
        DerbyUtil.shutdownDerbyNetworkServer();
        return klasy;
    }

    public static void startClass(LinkedHashMap<String, byte[]> mapa, String className) {
        ClassLoader loader = new AriaClassLoader(mapa);
        Set<Map.Entry<String, byte[]>> set = mapa.entrySet();
        for (Map.Entry<String, byte[]> me : set) {
            try {
                loader.loadClass(me.getKey());
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            }
        }
        try {
            try {
                Class.forName(className, true, loader)
                        .getDeclaredMethod("main", new Class[]{String[].class})
                        .invoke(null, new Object[]{null});
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            }
        } catch (IllegalArgumentException | NoSuchMethodException | InvocationTargetException | IllegalAccessException |
                 SecurityException e) {
            e.printStackTrace();
        }
        //-
    }

    public static void sourceFilesToDerby(String dir, String databaseName) {
        DerbyUtil.startDerbyNetworkServer("localhost", 1527);
        DerbyUtil.startDerbyEngine("ClientDriver");
        Connection con = DerbyUtil.connectServerDB(databaseName, "localhost",
                "1527", "create=true");
        Statement stat = null;
        try {
            stat = con.createStatement();
            stat.executeUpdate(DerbyUtil.createTableSource);
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            DerbyUtil.close(stat);
        }
        PreparedStatement pstm = null;
        File file;
        long length;
        BufferedReader br = null;
        String[] klasy = listFileNames(dir, ".java");
        try {
            pstm = con.prepareStatement("INSERT INTO klasy VALUES(?,?)");
            for (String s : klasy) {
                file = new File("assets/klasy/" + s);
                try {
                    br = new BufferedReader(new FileReader(file));
                } catch (FileNotFoundException e1) {
                    e1.printStackTrace();
                }
                length = file.length();
                try {
                    pstm.setString(1, s);
                    pstm.setClob(2, br, length);
                    pstm.execute();
                    //pstm.clearParameters();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        //-
        DerbyUtil.close(pstm);
        DerbyUtil.close(br);
        DerbyUtil.close(con);
        DerbyUtil.shutdownDerbyEngine();
        DerbyUtil.shutdownDerbyNetworkServer();
    }

    public static void sourceFilesFromDerby() {
        DerbyUtil.startDerbyNetworkServer("localhost", 1527);
        DerbyUtil.startDerbyEngine("ClientDriver");
        Connection con = DerbyUtil.connectServerDB("klasy", "localhost", "1527",
                "");
        Statement stat = null;
        ResultSet rs = null;
        InputStream in = null;
        FileOutputStream fos = null;
        int count = 0;
        try {
            stat = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
                    ResultSet.CONCUR_READ_ONLY);
            rs = stat.executeQuery("SELECT COUNT(*) FROM klasy");
            while (rs.next()) {
                count = rs.getInt(1);
            }
        } catch (SQLException e1) {
            e1.printStackTrace();
        }
        File[] files = new File[count];
        int i = 0;
        try {
            stat = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
                    ResultSet.CONCUR_READ_ONLY);
            rs = stat.executeQuery("SELECT * FROM klasy");
            rs.beforeFirst();
            while (rs.next()) {
                String str = rs.getString("plik");
                Blob blob = rs.getBlob("klasa");
                in = blob.getBinaryStream();
                File file = new File(str);
                try {
                    fos = new FileOutputStream(file);
                } catch (FileNotFoundException e) {
                    e.printStackTrace();
                }
                copyStream(in, fos);
                files[i] = file;
                i++;
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            DerbyUtil.close(stat);
            DerbyUtil.close(rs);
            close(in);
            close(fos);
        }
        //-
        DerbyUtil.close(con);
        DerbyUtil.shutdownDerbyEngine();
        DerbyUtil.shutdownDerbyNetworkServer();
    }

    public static void copyStream(InputStream in, OutputStream out) {
        byte[] buffer = new byte[1024];
        while (true) {
            int bytesRead = -1;
            try {
                bytesRead = in.read(buffer);
            } catch (IOException e) {
                e.printStackTrace();
            }
            if (bytesRead == -1) {
                break;
            }
            try {
                out.write(buffer, 0, bytesRead);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    //---------------------------------- R115 ----------------------------------------------
    public static TreeMap<String, String> readFile(String file) {
        FileReader fr = null;
        BufferedReader br = null;
        TreeMap<String, String> map = new TreeMap<>();
        String line;
        try {
            fr = new FileReader(file);
            br = new BufferedReader(fr);
            while ((line = br.readLine()) != null) {
                line = line.replace('"', ' ');
                int pos = line.indexOf(";");
                String key = line.substring(0, pos);
                String value = line.substring(pos + 1);
                map.put(key.trim(), value.trim());
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (br != null) {
                try {
                    br.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (fr != null) {
                try {
                    fr.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return map;
    }
}
